#include "legato.h"
#include <python2.7/Python.h>

#define ASSERT_NOTNULL(x)	{ LE_ASSERT ( x != NULL ); }
#define ASSERT_0(Val)		{ LE_FATAL_IF ( Val != 0, "%d!=0", Val ); }

#define FD_MODE_R			"r"
#define FD_MODE_RW			"r+"
#define FD_MODE_W			"w"
#define FD_MODE_CREATERW	"w+"
#define FD_MODE_APPEND		"a"
#define FD_MODE_RAPPEND		"a+"

void fd_StrmOpen ( const char* sFile, const char* sMode, FILE** ppFile )
{
	*ppFile = fopen ( sFile, sMode );
	LE_FATAL_IF ( *ppFile == NULL, "\"%s\" %s strm=%p %s", sFile, sMode, *ppFile, strerror ( errno ) );

	LE_INFO ( "%s %s strm=%p", sFile, sMode, *ppFile );
}
void fd_StrmClose ( FILE** ppFile )
{
	int	nRet;

	nRet = fclose ( *ppFile );
	LE_FATAL_IF ( nRet != 0, "%d strm=%p %s", nRet, *ppFile, strerror ( errno ) );

	LE_INFO ( "strm=%p", *ppFile );

	*ppFile = NULL;
}

static void fOnExit ( void )
{
	LE_DEBUG ( "fOnExit" );
}
void py_Init ( int argc, char** argv )
{
	const char*	sMisc = NULL;
	int			nRet;

	Py_Initialize ( );

	nRet = Py_AtExit ( fOnExit );
	ASSERT_0 ( nRet );

	sMisc = Py_GetProgramName ( );
	ASSERT_NOTNULL ( sMisc );
	LE_DEBUG ( "%s", sMisc );

	sMisc = Py_GetPrefix ( );
	ASSERT_NOTNULL ( sMisc );
	LE_DEBUG ( "prefix: %s", sMisc );

	sMisc = Py_GetExecPrefix ( );
	ASSERT_NOTNULL ( sMisc );
	LE_DEBUG ( "exec prefix: %s", sMisc );

	sMisc = Py_GetVersion ( );
	ASSERT_NOTNULL ( sMisc );
	LE_DEBUG ( "%s", sMisc );

	sMisc = Py_GetPlatform ( );
	ASSERT_NOTNULL ( sMisc );
	LE_DEBUG ( "Platform: %s", sMisc );

	sMisc = Py_GetCopyright ( );
	ASSERT_NOTNULL ( sMisc );
	LE_DEBUG ( "%s", sMisc );

	sMisc = Py_GetCompiler ( );
	ASSERT_NOTNULL ( sMisc );
	LE_DEBUG ( "%s", sMisc );

	sMisc = Py_GetBuildInfo ( );
	ASSERT_NOTNULL ( sMisc );
	LE_DEBUG ( "Build: %s", sMisc );
}
void py_Exit ( void )
{
	Py_Finalize ( );
}

void py_RunStr ( const char* sCmd )
{
	int	nRet;

	nRet = PyRun_SimpleString ( sCmd );
	ASSERT_0 ( nRet );
}

void py_RunFile ( const char* sFile )
{
	int		nRet;
	FILE*	pFile = NULL;
	
	fd_StrmOpen ( sFile, FD_MODE_R, & pFile );
	nRet = PyRun_SimpleFile ( pFile, sFile );
	ASSERT_0 ( nRet );
	fd_StrmClose ( & pFile );
}

COMPONENT_INIT
{
	LE_INFO ( "Build >= " __DATE__ " " __TIME__ "" );

	py_Init ( 0, NULL );
	py_RunStr (
		"from time import time,ctime\n"
		"print 'Today is',ctime(time())\n"
	);

	py_RunFile ( "/usr/share/myScripts/Test01.py" );
	py_Exit ( );
}
